﻿using GraphicsManager.Enums;
using GraphicsManager.Objects;
using OpenTK.Mathematics;
using OpenTK.Windowing.Common.Input;

namespace GraphicsManager.Interfaces;

public interface IRenderObject
{
	public void ForceDistanceUpdate();
	public void ForceDistanceUpdate(IParent parent);
	public BetterContextMenu? ContextMenu { get; set; }
	public ObjectAnchor Anchor { get; set; }
	public bool MouseInside { get; set; }
	public bool IgnoreHover { get; set; }
	public bool AllowHoverFromBehind { get; set; }
	public bool Loaded { get; }
	public void LoadToParent(IParent Parent, IWindow Window);
	public void Draw(int x, int y, int w, int h);
	public MouseCursor HoverMouse { get; set; }
	public void Clean();
	public void Focus();
	public void UnFocus();
	public Vector2i Size { get; set; }
	public Vector3i Location { get; set; }
	public Vector2 SizeAsFloat { get; }
	public Vector2 LocationAsFloat { get; }
	public Vector2i Distance { get; set; }
	public IParent? Parent { get; }
	public IWindow? Window { get; }
	public bool Visible { get; set; }
	public object? Tag { get; set; }

	public event Func<IRenderObject, Task>? Clicked;
	public event Func<IRenderObject, Task>? MouseLeave;
	public event Func<IRenderObject, Task>? MouseEnter;
	public event Func<IRenderObject, Task>? SizeChanged;
	public event Func<IRenderObject, Task>? WindowLoaded;
}
