using GraphicsManager.Interfaces;
using GraphicsManager.Objects.Core;
using OpenTK.Mathematics;

namespace GraphicsManager.Objects;

public class BetterContextMenu : UserControl
{
    public Vector4i Margins
    {
        get
        {
            return mg;
        }
        set
        {
            mg = value;
            BlockDraw = true;
            fl.Location = new(mg.W, mg.X, Location.Z);
            if (AutoSizeWidth || AutoSizeHeight)
            {
                int x = 0, y = 0;
                if (AutoSizeWidth && AutoSizeHeight)
                {
                    for (int i = 0; i < Controls.Length; i++)
                    {
                        y += Controls[i].Size.Y;
                        x += Controls[i].Size.X + fl.SpaceBetweenObjects;
                    }
                    
                    x -= fl.SpaceBetweenObjects;
                    fl.Size = new(x, y);
                    Size = new(mg.Y + mg.W + x, mg.X + mg.Z + y);
                }
                else if (AutoSizeWidth)
                {
                    for (int i = 0; i < Controls.Length; i++)
                    {
                        x += Controls[i].Size.X + fl.SpaceBetweenObjects;
                    }

                    x -= fl.SpaceBetweenObjects;
                    fl.Size = new(x, fl.Size.Y);
                    Size = new(mg.Y + mg.W + x, Size.Y);
                }
                else
                {
                    for (int i = 0; i < Controls.Length; i++)
                    {
                        y += Controls[i].Size.Y;
                    }

                    fl.Size = new(fl.Size.X, y);
                    Size = new(Size.X, mg.X + mg.Z + y);
                }
            }

            BlockDraw = false;
            TryDraw();
        }
    }
    private FlowLayout fl = new()
    {
        BackgroundColor = Color4.Transparent
    };
    
    private Vector4i mg;
    
    public int SpaceBetweenObjects
    {
        get => fl.SpaceBetweenObjects;
        set => fl.SpaceBetweenObjects = value;
    }
    
    public void ShowContext(IWindow ParentWindow)
    {
        if (!Visible)
        {
            if (Parent is null)
            {
                ParentWindow.Controls.Add(this);
                LoadToParent(ParentWindow, ParentWindow);
            }
        }
        if (Size.Y + (int)ParentWindow.MousePosition.Y > ParentWindow.Size.Y) Location = new((int)ParentWindow.MousePosition.X, ParentWindow.Size.Y - Size.Y, Location.Z);
        else Location = new((int)ParentWindow.MousePosition.X, (int)ParentWindow.MousePosition.Y, Location.Z);
        Visible = true;
        TryDraw();
    }
    
    public void HideContext(IWindow Parent)
    {
        if (!Visible) return;
        Visible = false;
        TryDraw();
    }

    public bool AutoSizeWidth { get; set; } = true;
    public bool AutoSizeHeight { get; set; } = true;


    public BetterContextMenu()
    {
        Controls.Add(fl);
        base.Visible = false;
    }
    
    public BetterContextMenu(Texture t)
    :base(t)
    {
        Controls.Add(fl);
        base.Visible = false;
    }

    public void AddLine()
    {
        AddLine(new(255,255,255,255));
    }
    
    public void AddLine(Color4 color, int Height = 1)
    {
        Rectangle r = new()
        {
            BackgroundColor = color,
            Size = new(fl.Size.X, Height)
        };
        AddOption(r);
    }

    public void AddLabel(ILabel label)
    {
        AddOption(label);
    }
    
    public Label AddLabel(FontInteraction fi, string text, Color4 color)
    {
        Label l;
        AddLabel(l=new Label(fi){ Text = text, Color = color});
        return l;
    }

    public void AddOption(IRenderObject obj)
    {
        int x = fl.Size.X, y = fl.Size.Y;
        if (AutoSizeWidth && obj.Size.X > fl.Size.X)
        {
            Size = new(obj.Size.X + mg.Y + mg.W, Size.Y);
            x = obj.Size.X;
        }
        
        if (AutoSizeHeight)
        {
            Size = new(Size.X, Size.Y + SpaceBetweenObjects + obj.Size.Y);
            y += obj.Size.Y + SpaceBetweenObjects;
        }

        fl.Size = new(x, y);
        fl.Controls.Add(obj);
    }
}