using GraphicsManager.Interfaces;
using GraphicsManager.Objects.Core;
using OpenTK.Mathematics;

namespace GraphicsManager.Objects;

public class RoundedButton : Rectangle
{
	internal Label _label;
	public RoundedButton(Texture t, FontFamily family)
	:base(t)
	{
		_label = new Label(family);
	}
	
	public RoundedButton(Texture t, FontInteraction family)
	:base(t)
	{
		_label = new Label(family);
	}
	
	public FontInteraction Font { get => _label.Font; }
	public string Text { get => _label.Text; set => _label.Text = value; }
	public override Vector2i Size
	{
		get
		{
			return base.Size;
		}
		set
		{
			base.Size = value;
			_label.Location = new(Location.X + (((value.X) / 2) - (_label.Size.X / 2)), Location.Y + (((value.Y) / 2) - (_label.Size.Y / 2)), Location.Z);
		}
	}
	
	public override Vector3i Location { 
		get => base.Location; 
		set
		{
			base.Location = value;
			_label.Location = new(value.X + (((Size.X) / 2) - (_label.Size.X / 2)), value.Y + (((Size.Y) / 2) - (_label.Size.Y / 2)), value.Z);
		}
	}
	
	public Color4 FontColor { get => _label.Color; set => _label.Color = value; }
	public override bool Visible
	{
		get => base.Visible;
		set 
		{ 
			_label.Visible = value;
			base.Visible = value;
		}
	}

	public override void Clean()
	{
		_label.Clean();
		base.Clean();
	}

	public override void Draw(int x, int y, int w, int h)
	{
		if (!Visible || !Loaded) return;
		base.Draw(x,y,w,h);
		_label.Draw(x,y,w,h);
	}

	public override void LoadToParent(IParent Parent, IWindow Window)
	{
		if (Loaded) return;
		base.LoadToParent(Parent, Window);
		_label.LoadToParent(Parent, Window);
		Location = Location;
	}
}
