using System.Runtime.CompilerServices;
using System.Runtime.Intrinsics.Arm;
using System.Security.Cryptography;
using GraphicsManager.Interfaces;
using GraphicsManager.Structs;
using OpenTK.Windowing.Desktop;

namespace GraphicsManager.Objects.Core;

public class TextureManager
{
    public static readonly Dictionary<IGLFWGraphicsContext, TextureManager> TextureManagers = new();

    private List<Texture> _textures = new();
    public IWindow Window { get; private set; }

    public static TextureManager GetTextureManager(IGLFWGraphicsContext context, IWindow win)
    {
        if (context is null) throw new ArgumentNullException(nameof(context));
        if (!TextureManagers.ContainsKey(context))
        {
            TextureManager tmp = new(context);
            tmp.Window = win;
            TextureManagers.Add(context, tmp);
            return tmp;
        }

        return TextureManagers[context];
    }

    public IGLFWGraphicsContext Context { get; private set; }

    private TextureManager(IGLFWGraphicsContext context)
    {
        Context = context;
    }

    public IReadOnlyList<Texture> Textures { get => _textures.AsReadOnly(); }

    public void RemoveTexture(Texture Texture)
    {
        _textures.Remove(Texture);
    }
    
    public Texture AddTexture(Stream TextureStream, bool UseMissing = true)
    {
        Context.MakeCurrent();
        Texture t = Texture.GetTexture(TextureStream, this, UseMissing);
        _textures.Add(t);
        return t;
    }
    /*
    public Texture AddTexture(byte[] Texture, bool Rounded = false)
    {
        List<byte> hash2 = SHA256.HashData(Texture).ToList();
        hash2.Insert(0, Convert.ToByte(Rounded));
        byte[] hash = hash2.ToArray();
        if (TextureHashMap.ContainsKey(hash)) return TextureHashMap[hash];
        Context.MakeCurrent();
        TextureHashMap.Add(hash, new(Texture, Rounded));
        return TextureHashMap[hash];
    }*/
}