using GraphicsManager.Enums;
using GraphicsManager.Objects.Core;
using OpenTK.Mathematics;

namespace GraphicsManager.Objects;

public class ProgressBar : ParentBase
{
    private Rectangle ProgresRct;

    public Color4 ProgressColor
    {
        get
        {
            return ProgresRct.BackgroundColor;
        }
        set
        {
            ProgresRct.BackgroundColor = value;
        }
    }
    
    public bool UpdateOnDraw { get; set; }
    
    public ProgressBar()
    {
        ProgresRct = new()
        {
            Location = new((int)gap, (int)gap, 0),
            IgnoreHover = true,
            BackgroundColor = Color4.Green,
            Anchor = ObjectAnchor.All
        };
        Controls.Add(ProgresRct);
    }
    
    public ProgressBar(Texture Background, Texture Progress)
    :base(Background)
    {
        ProgresRct = new(Progress)
        {
            Location = new((int)gap, (int)gap, 0),
            IgnoreHover = true,
            BackgroundColor = Color4.Green,
            TextureDisplay = TextureDisplay.ProgressHorizontalCenter,
            Anchor = ObjectAnchor.All
        };
        TextureDisplay = TextureDisplay.VerticalCenter;
        Controls.Add(ProgresRct);
    }
    
    public ProgressBar(Texture Background)
        :base(Background)
    {
        ProgresRct = new(Background)
        {
            Location = new((int)gap, (int)gap, 0),
            IgnoreHover = true,
            BackgroundColor = Color4.Green,
            TextureDisplay = TextureDisplay.ProgressHorizontalCenter,
            Anchor = ObjectAnchor.All,
        };
        TextureDisplay = TextureDisplay.VerticalCenter;
        Controls.Add(ProgresRct);
    }

    public Shader InnerShader
    {
        get
        {
            return ProgresRct.Shader;
        }
        set
        {
            ProgresRct.Shader = value;
        }
    }

    public override Vector2i Size
    {
        get
        {
            return base.Size;
        }
        set
        {
            base.Size = value;
            if (!UpdateOnDraw) UpdateProgress();
        }
    }

    public override Vector3i Location
    {
        get
        {
            return base.Location;
        }
        set
        {
            Vector3i diff = value - base.Location;
            ProgresRct.Location -= diff;
            base.Location = value;
        }
    }

    private ulong mpv = 100, pv = 0;
    private uint gap = 5;

    public ulong MaxProgressValue
    {
        get
        {
            return mpv;
        }
        set
        {
            mpv = value;
            if (!UpdateOnDraw) UpdateProgress();
        }
    }
    
    public ulong ProgressValue
    {
        get
        {
            return pv;
        }
        set
        {
            if (value > MaxProgressValue) pv = MaxProgressValue;
            else pv = value;
            if (!UpdateOnDraw) UpdateProgress();
        }
    }
    
    public uint ProgressGap
    {
        get
        {
            return gap;
        }
        set
        {
            gap = value;
            if (!UpdateOnDraw) UpdateProgress();
        }
    }

    public override void Draw(int x, int y, int w, int h)
    {
        if (UpdateOnDraw) UpdateProgress();
        base.Draw(x, y, w, h);
    }

    public void UpdateProgress()
    {
        double percent = (double)ProgressValue / MaxProgressValue;
        long MaxPixels = Size.X - gap - gap;
        if (ProgresRct.Location.X != gap)
        {
            ProgresRct.Location = new((int)gap, (int)gap, 0);
            ProgresRct.Size = new((int)(MaxPixels * percent), (int)(Size.Y - gap - gap));
        }
        else ProgresRct.Size = new((int)(MaxPixels * percent), ProgresRct.Size.Y);
    }
}