using GraphicsManager.Enums;
using GraphicsManager.Interfaces;
using GraphicsManager.Objects.Core;
using OpenTK.Mathematics;
using OpenTK.Windowing.Common;
using OpenTK.Windowing.Common.Input;

namespace GraphicsManager.Objects;

public class TabControl : ParentBase, IParent
{
	private Texture t;
	public TabControl(Texture tex, FontFamily fam)
	{
		t = tex;
	    TitleFont = FontInteraction.Load(fam);
    }
    
    public TabControl(Texture tex, FontInteraction fam)
    {
	    t = tex;
	    TitleFont = fam;
    }

    private ControlList Buttonts = new();

    public void AddPage(string Title, UserControl page)
    {
	    RoundedButton tmp;
	    uint loc = (uint)Buttonts.Length;
	    int locc = 0, loccc = 0;
	    if (loc != 0)
	    {
		    locc = Buttonts[loc - 1].Location.X;
		    loccc = Buttonts[loc - 1].Size.X;
	    }
	    Buttonts.Add(tmp = new RoundedButton(t, TitleFont)
	    {
		    Location = new(locc+ loccc + (TabSpace * (int)loc) + Border, Border, Location.Z),
		    Text = Title,
		    Tag = loc,
		    FontColor = this.TextColor
	    });
	    
	    if (loc == PageIndex) tmp.BackgroundColor = this.SelectedColor;
	    tmp.Clicked += TmpOnClicked;
	    tmp.Size = new(tmp._label.Size.X + ((TextBorder + 4) * 2), tmp._label.Size.Y + ((TextBorder + 4) * 2));
	    page.Anchor = ObjectAnchor.All;
	    page.Location = new(Border, tmp.Location.Y + tmp.Size.Y + TabSpace, page.Location.Z);
	    page.Size = new(Size.X - Border - Border, Size.Y - page.Location.Y - Border);
	    if (PageIndex != loc) page.Visible = false;
	    Controls.Add(page);
    }
    
    public void AddPage(string Title, FlowLayout page)
    {
	    RoundedButton tmp;
	    uint loc = (uint)Buttonts.Length;
	    int locc = 0, loccc = 0;
	    if (loc != 0)
	    {
		    locc = Buttonts[loc - 1].Location.X;
		    loccc = Buttonts[loc - 1].Size.X;
	    }
	    Buttonts.Add(tmp = new RoundedButton(t, TitleFont)
	    {
		    Location = new(locc+ loccc + (TabSpace * (int)loc) + Border, Border, Location.Z),
		    Text = Title,
		    Tag = loc,
		    FontColor = this.TextColor,
		    Anchor = ObjectAnchor.Top | ObjectAnchor.Left
	    });
	    
	    if (loc == PageIndex) tmp.BackgroundColor = this.SelectedColor;
	    tmp.Clicked += TmpOnClicked;
	    tmp.Size = new(tmp._label.Size.X + ((TextBorder + 4) * 2), tmp._label.Size.Y + ((TextBorder + 4) * 2));
	    page.Anchor = ObjectAnchor.All;
	    page.Location = new(Border, tmp.Location.Y + tmp.Size.Y + TabSpace, page.Location.Z);
	    page.Size = new(Size.X - Border - Border, Size.Y - page.Location.Y - Border);
	    if (PageIndex != loc) page.Visible = false;
	    Controls.Add(page);
    }

    private Task TmpOnClicked(IRenderObject arg)
    {
	    PageIndex = (uint)arg.Tag!;
	    return Task.CompletedTask;
    }

    public Color4 TextColor { get; set; } = new(255, 255, 255, 255);
    public Color4 BorderColor { get; set; } = new(40, 40, 40, 255);
    public Color4 SelectedColor { get; set; } = Color4.DarkCyan;
    public Color4 InsideColor { get; set; } = new(0, 0, 0, 255);
    public int TextBorder { get; set; } = 5;
    public int TabSpace { get; set; } = 5;
    public int Border { get; set; } = 10;
    public FontInteraction TitleFont { get; set; } = null!;

    public override bool Visible
    {
	    get => base.Visible;
	    set
	    {
		    BlockDraw = true;
		    base.Visible = value;
		    for (int i = 0; i < Buttonts.Length; i++)
			    Buttonts[i].Visible = value;
		    if (value) PageIndex = PageIndex;
		    else
		    {
			    for (int i = 0; i < Controls.Length; i++)
				    Controls[i].Visible = value;
		    }
		    if (Parent is not null) Parent.TryDraw();
		    BlockDraw = false;
	    }
    }
    private uint pgi = 0;
    public uint PageIndex
    {
	    get
	    {
		    return pgi;
	    }
	    set
	    {
		    pgi = value;
		    BlockDraw = true;
		    if (base.Visible)
		    {
			    for (int i = 0; i < Buttonts.Length; i++)
				    ((RoundedButton)Buttonts[i]).BackgroundColor = BorderColor;
			    ((RoundedButton)Buttonts[value]).BackgroundColor = SelectedColor;
			    for (int i = 0; i < Controls.Length; i++)
				    Controls[i].Visible = false;
			    Controls[value].Visible = true;
		    }

		    if (Parent is not null) Parent.TryDraw();
		    BlockDraw = false;
	    }
    }
    
    public override void ParentResize(ResizeEventArgs e)
    {
	    BlockDraw = true;
	    if (e.Width == 0 && e.Height == 0) return;
	    if (!(Controls.Length >= PageIndex)) return;
	    if (!Controls[PageIndex].Loaded) return;
	    bool top = (Controls[PageIndex].Anchor & ObjectAnchor.Top) == ObjectAnchor.Top;
	    bool left = (Controls[PageIndex].Anchor & ObjectAnchor.Left) == ObjectAnchor.Left;
	    bool right = (Controls[PageIndex].Anchor & ObjectAnchor.Right) == ObjectAnchor.Right;
	    bool bottom = (Controls[PageIndex].Anchor & ObjectAnchor.Bottom) == ObjectAnchor.Bottom;
	    if (!top && !bottom) { Controls[PageIndex].Anchor |= ObjectAnchor.Top; top = true; }
	    if (!left && !right) { Controls[PageIndex].Anchor |= ObjectAnchor.Left; left = true; }
	    int lx = (left ? Controls[PageIndex].Location.X : Size.X - Controls[PageIndex].Distance.X - Controls[PageIndex].Size.X);
	    int ly = (top ? Controls[PageIndex].Location.Y : Size.Y - Controls[PageIndex].Distance.Y - Controls[PageIndex].Size.Y);
	    int sy = (bottom ? Size.Y - Controls[PageIndex].Distance.Y - ly : Controls[PageIndex].Size.Y);
	    int sx = (right ? Size.X - Controls[PageIndex].Distance.X - lx : Controls[PageIndex].Size.X);
	    Controls[PageIndex].Size = new(sx, sy);
	    Controls[PageIndex].Location = new(lx, ly, Controls[PageIndex].Location.Z);
	    if (Controls[PageIndex] is IParent parent)
	    {
		    parent.ParentResize(e);
	    }
	    for (int i = 0; i < Buttonts.Length; i++)
	    {
		    if (!Buttonts[i].Loaded) continue;
		    top = (Buttonts[i].Anchor & ObjectAnchor.Top) == ObjectAnchor.Top;
		    left = (Buttonts[i].Anchor & ObjectAnchor.Left) == ObjectAnchor.Left;
		    right = (Buttonts[i].Anchor & ObjectAnchor.Right) == ObjectAnchor.Right;
		    bottom = (Buttonts[i].Anchor & ObjectAnchor.Bottom) == ObjectAnchor.Bottom;
		    if (!top && !bottom) { Buttonts[i].Anchor |= ObjectAnchor.Top; top = true; }
		    if (!left && !right) { Buttonts[i].Anchor |= ObjectAnchor.Left; left = true; }
		    lx = (left ? Buttonts[i].Location.X : Size.X - Buttonts[i].Distance.X - Buttonts[i].Size.X);
		    ly = (top ? Buttonts[i].Location.Y : Size.Y - Buttonts[i].Distance.Y - Buttonts[i].Size.Y);
		    sy = (bottom ? Size.Y - Buttonts[i].Distance.Y - ly : Buttonts[i].Size.Y);
		    sx = (right ? Size.X - Buttonts[i].Distance.X - lx : Buttonts[i].Size.X);
		    Buttonts[i].Size = new(sx, sy);
		    Buttonts[i].Location = new(lx, ly, Buttonts[i].Location.Z);
		    if (Buttonts[i] is IParent parent2)
		    {
			    parent2.ParentResize(e);
		    }
	    }
	    Parent!.TryDraw();
	    BlockDraw = false;
    }

    public override void LoadToParent(IParent Parent, IWindow Window)
    {
    	if (Loaded) return;
    	BlockDraw = true;
        if (!Window!.Context.IsCurrent) Window.Context.MakeCurrent();
    	base.LoadToParent(Parent, Window);
        for (int i = 0; i < Buttonts.Length; i++)
        {
	        Buttonts[i].LoadToParent(this, Window);
        }

    	BlockDraw = false;
    }

    public override void Draw(int x, int y, int w, int h)
    {
    	if (Loaded && Visible)
    	{
    		base.Draw(x,y,w,h);
            if (!(Controls.Length >= (PageIndex))) return;
            if (!Controls[PageIndex].Loaded) return;
            Controls[PageIndex].Draw(x,y,w,h);
            for (int i = 0; i < Buttonts.Length; i++) 
	            Buttonts[i].Draw(x,y,w,h);
        }
    }
    
    public override void Clean()
    {
	    for (int i = 0; i < Buttonts.Length; i++)
	    {
		    Buttonts[i].Clean();
	    }
    	base.Clean();
    }
}